                   TITLE   FMONITOR
PAGE, 132
COMMENT |

       Copyright by John Poindexter  Rockville, MD
       Version 1.03  December 14, 1988

       xMONITOR Series

       This is one of a family of MONITOR programs designed to work with
       applications running in DESQview windows which are not DESQview
       applications, but for which you want to perform some function related
       to their presence in DESQview.

       FMONITOR is a DESQview program to be used with DESQview,
       WordPerfect 5.0, and Hercules Graphics Card Plus when using the
       RamFont features built into WP.  It monitors whether WordPerfect is
       in the foreground or background and changes the HGC+ mode
       accordingly.  If WP is in the foreground and not in graphics mode,
       it puts the HGC+ in the 48k RamFont text, 9 dot, 48k RamFont
       attribute mode.  If the window is in background, it resets the HGC+
       to normal text mode unless the foreground program is in graphics
       mode.  It assumes that WordPerfect uses a full screen and the
       foreground program covers location 0,0.  It also assumes that
       DESQview will take care of changes to the various graphic modes of
       the HGC+.

       xMONITORs use the DESQview API.

       Usage:  You must first install the FMONITOR programs in DESQview by
       adding the FM-PIF.DVP file to the DESQview window menu by following
       the standard procedures for adding programs.  Be sure to use FM-PIF.DVP
       since the window must run in background and the window size and memory
       have been reduced to the minimum.  After it is installed you can change
       the command FM for opening the window to whatever you want and you
       will need to change the Parameters line to agree with your location of
       the WordPerfect PIF file, WP-PIF.DVP.

       Syntax:   FMONITOR pif_filespec hgc_mode font_file open_win
                                      ^        ^         ^
                             (must have 1 and only 1 space at these points)

       where pif_filespec is the d:\path\xx-PIF.DVP for the main application
       being monitored and hgc_mode is the HGC+ mode you want the application
       to use.  The hgc_mode numbers are the XMODEREG numbers from page 70
       of the HGC+ manual.  WordPerfect uses hgc_mode 5.  The font_file is
       either 2 or 3 corresponding to the option numbers on the WP menu.
       Font_file 2 is the 12 x 256 character file, HRF12.FRS.  Font_file 3
       is the 6 x 512 character file, HRF6.FRS.  Open_win is either Y or N
       (you must use upper case) where Y means that FM will open the WP
       window.  N means that FM will not open WP window and FM must be
       opened when WP is zoomed to full screen and in the foreground.

       FMONITOR checks for the presence of both DESQview and the HGC+ GB112
       board and will not load unless both are present.  It is designed to
       work with DESQview version 2.01.

       To terminate FMONITOR just close its window.

       The author has found that using Int 10h to get the current video
       mode is unreliable in a multi-tasking environment at least with the
       Hercules card.  Apparently some programs such as Lotus 123 and
       WordPerfect do not update the BIOS data area to reflect the correct
       video mode when they change the Hercules modes.  Therefore this
       program uses DESQview to notify of mode changes and current mode.
|
;*************************************************************
;            PROGRAM EQUATES AND MACROS
;*************************************************************
;
include dvapi.inc
;
;GB112 constants
                                           ;
dmc_port           equ     03b8h           ; HGC+ display mode control port
status_port        equ     03bah           ; HGC+ display status port
config_port        equ     03bfh           ; HGC+ configuration switch
index_reg          equ     03b4h           ; 6856 index (pointer) register
data_reg           equ     03b5h           ; 6845 data register

scrn_off_text      equ     20h             ; bit masks for dmc_port
scrn_on_b          equ     28h             ; screen on blink enable set
scrn_on_nb         equ     08h             ; screen on blink enable clear
scrn_off_grph      equ     82h
scrn_on_grph       equ     8ah
                                           ;
id_mask            equ     00110000b       ; GB112 detection masks
id_code            equ     00010000b
                                           ;
text_buf_seg       equ     0b000h          ; video buffer start
ramfont_seg        equ     0b400h          ; RamFont character storage start
lramfont           equ     48*1024         ; 48k of RamFont storage
gpage_1            equ     0b800h          ; Graphics page 1 start
                                           ;
xModeReg           equ     14h             ; HGC+ psuedo 6845 RamFont control regs
scorereg           equ     15h
strikereg          equ     16h
                                           ;
blank_cnt          equ     2000h           ; Used to clear video buffer
                                           ;
pif                struc                   ; for DESQview 2.0+
                   db      36 dup (0)
path_cmd           db      64 dup (0)
                   db      316 dup (0)
pif                ends


;*************************************************************
;        CODE SEGMENT BEGINS
;*************************************************************
code_seg           SEGMENT
                   ASSUME  cs:code_seg
                   ORG     100h            ;ORG = 100h to make this into a
                                           ;".com" file
first:             jmp     load_monitor
                                           ;
;*************************************************************
;        PROGRAM DATA
;*************************************************************
                                           ;
                   DB      'FMONITOR.COM'
                   DB      'Copyright by John Poindexter'
                   DB      'Version 1.03'
                   DB      '14DEC88'
                                           ;
crt_cols           dw      ?               ;either 80 or 90 columns wide
count              db      0
                                           ;
; 6845 data tables
nine_by_14         db      61h,50h,52h,0fh,19h,6,19h,19h,2,0dh,0bh,0ch,0,0
eight_by_16        db      109,90,92,15,22,2,21,22,2,0fh,0dh,0eh,0,0
eight_by_14        db      109,90,92,15,25,6,25,25,2,0dh,09h,0ah,0,0
eight_by_8         db      109,90,92,15,45,2,43,44,2,7,7,6,0,0
;
;
cmd_tail           equ     80h
PSP_seg            dw      ?
;
;DESQview data
main_mailbox       dd      0
main               label   dword
main_win           dw      2 dup (0)
pif_buffer         pif     <>
;
;HGC+ data
hgc_mode        db      0
ramfont         db      5 ;48k RamFont text, 9 dot, 48k RamFont text attributes
textfont        equ     0
graph_mode      db      0
text_mode       equ     7
video_mode      db      7
font_file       db      2   ;use font_file2 or font_file3
font_file2      db      'HRF12.FRS',0
lfont_file2     equ     $-font_file2
font_file3      db      'HRF6.FRS',0
lfont_file3     equ     $-font_file3
open_window     db      'Y'
;
install_message db      0dh,0ah,'FMONITOR Version 1.03 installed.',0dh,0ah,'$'
fail_message1   db      0dh,0ah,'ERROR - Hercules Graphics Card not present '
                db      'or not presently activated.',0dh,0ah,'$'
fail_message2   db      0dh,0ah,'ERROR - Hercules Graphics Card Plus'
                db      ' (model GB112) not detected in system.',0dh,0ah,'$'
fail_message3   db      0dh,0ah,'ERROR - DESQview is not present.',0dh,0ah,'$'
file_error_msg  db      0dh,0ah,'ERROR - Unable to open PIF file.',0dh,0ah,'$'
path_error_msg  db      0dh,0ah,'ERROR - Path not in PIF file.',0dh,0ah,'$'
win_error_msg   db      0dh,0ah,'ERROR - Main application must be in full '
                db      'screen.',0dh,0ah,'$'
font_error_msg  db      0dh,0ah,'ERROR - Unable to open font file.',0dh,0ah,'$'
pause_msg       db      'Press any key to exit...','$'
mode_chg_msg    db      1bh,10h             ;manager stream ID
                dw      lmode_chg_msg-4     ;number of bytes that follow
                db      4ch                 ;notify after video mode change
lmode_chg_msg   equ     $-mode_chg_msg
;
;*************************************************************
;                    MONITOR PROCEDURE
; This keeps track of whether the main application is in the
; foreground or background and sets the HGC+ mode properly.
;*************************************************************
monitor            proc    near
;
begin_monitor:     @call   beginc                ;make sure not sliced away

;is there any mail waiting -- only mail is video mode change
check_mail:        @send   sizeof,mailme
                   @pop    bxax
                   test    ax,ax
                   jz      check_mode            ;no mail
                   mov     cx,ax                 ;set counter to no. of msgs
                   mov     graph_mode,1          ;set flag, video mode change
read_mail:         @send   read,mailme           ;get the mail
                   @pop    bxax                  ;get length of msg
                   @pop    essi                  ;point to msg
                   loop    read_mail             ;find last msg
                   mov     al,byte ptr es:[si+1] ;get last video mode
                   mov     video_mode,al         ;save it

check_mode:        cmp     video_mode,text_mode  ;are we in text mode
                   je      what_00               ;yes, go for more checks
                   jmp     recycle               ;no, must be graphic mode
                                                 ;DESQview can handle

;check to see if main application window is at 0,0
;
what_00:           xor     bx,bx                 ;check location 0,0
                   mov     es,bx                 ;start at top of list
                   @call   locate
                   mov     ax,es                 ;get high order word of handle
                   test    ax,ax                 ;is it 0
                   jnz     check_main            ;no, check for main appl.
                   jmp     recycle               ;no window there so try again
check_main:        cmp     ax,main_win[2]        ;is it main application
                   je      check_font_mode       ;yes, go check HGC+ mode
                   jmp     check_text_mode
                                                 ;
check_font_mode:   mov     al,ramfont
                   cmp     hgc_mode,al           ;is HGC+ in RamFont mode
                   jne     set_ramfont_mode      ;no, go set it
                   jmp     recycle               ;already there
                                                 ;
check_text_mode:   cmp     hgc_mode,textfont     ;is HGC+ in normal text mode
                   jne     set_text_mode         ;no, go set it
                   jmp     recycle               ;already there
                                                 ;
set_ramfont_mode:  mov     crt_cols,80           ;assume 80 columns wide
                   xor     bx,bx                 ;set blanking to 0000 for 48k
                   mov     al,ramfont            ;see whether 8 or 9 dot
                   mov     hgc_mode,al           ;save value
                   xor     ah,ah
                   and     al,010h               ;AX now set for 8 or 9 dot
                   cmp     al,0                  ;check value for col width
                   je      set_6845              ;already set for 80
                   mov     crt_cols,90
                   jmp     set_6845              ;go set the controller
                                                 ;
set_text_mode:     mov     hgc_mode,textfont     ;set flag
                   xor     ax,ax                 ;normal text also 9 dot wide
                   mov     bx,0720h              ;set blanking to normal space
                                                 ;
;***************************************************************************
;                               PROG_6845
;
;       This section programs the 6845 for 9 x 14 matrix, 8 x 14 matrix
;       or 8 x 8  matrix depending on the contents of the AX register
;       ON ENTRY :
;       AX = 0 for 9x14, AX = 1 for 8x14, AX = 2 for 8x8, AX = 3 for 8x16
;       BX = the blanking value used to clear the text buffer
;               blanking value = 0720h for text mode and 4k ramfont mode
;               blanking value = 0000h for 48k ramfont mode.
;
;****************************************************************************
set_6845:
        push    bx                              ;save blanking value
                                                ;
;select the appropriate data table
                                                ;
        cmp     ax,1
        je      eight_dot
        cmp     ax,2
        je      eight_2
        cmp     ax,3
        je      eight_by_sixteen

        mov     si,offset nine_by_14
        jmp     screen_off

eight_dot:
        mov     si,offset eight_by_14
        jmp     screen_off

eight_2:
        mov     si,offset eight_by_8
        jmp     screen_off

eight_by_sixteen:
        mov     si,offset eight_by_16

screen_off:
        mov     al,scrn_off_text
        mov     dx,dmc_port
        out     dx,al

;program the 6845
        mov     dx,index_reg
        mov     cx,14

        xor     ah,ah                   ;start from register 0

loadtable:
        mov     al,ah
        out     dx,al                   ;output the register number

        inc     dx                      ;point to the data register
        lodsb                           ;load al with next table value

        out     dx,al                   ;output data value

        inc     ah                      ;point to next register
        dec     dx                      ;point to index register

        loop    loadtable               ;continue loading until done

;clear the text buffer, but leave the screen off
;DO NOT BLANK THE SCREEN FOR THIS APPLICATION
;setup_ptr:
;        mov     ax,text_buf_seg
;        mov     es,ax
;        xor     di,di
;        mov     cx,blank_cnt
;        pop     ax                      ;get blanking value
;clear the buffer
;rep     stosw
                                           ;
;screen is still off, select rom character generator
                                           ;
                   mov     dx,index_reg
                   mov     al,xModeReg
                   out     dx,al
                                           ;
                   mov     dx,data_reg
                   xor     ax,ax
                   mov     al,hgc_mode
                   out     dx,al
                                           ;
; update BIOS RAM area in segment 40h
                   push    ds
                   mov     ax,40h
                   mov     ds,ax
                   mov     ax,crt_cols
                   mov     word ptr ds:[4ah],ax
                   pop     ds
                                           ;
                   cmp     hgc_mode,textfont
                   jz      screen_on       ;yes, skip next section
;set the underline and overstrike positions
                   mov     dx,index_reg
                   mov     al,scorereg
                   out     dx,al

                   mov     dx,data_reg
                   mov     ax,13
                   out     dx,al            ;underscore at h - 1

                   mov     dx,index_reg
                   mov     al,strikereg
                   out     dx,al

                   mov     dx,data_reg
                   mov     ax,7
                   out     dx,al           ;overstrike at h/2
                                           ;
;reload fonts if have been in graphics mode
                   cmp     graph_mode,0
                   je      screen_on       ;no, no need to reload font
                   mov     graph_mode,0    ;reset graph flag
                   lea     dx,pif_buffer.path_cmd ;point to font_file
                   mov     ax,3d00h         ;open file for reading
                   int     21h
                   jnc     read_font_file
                   lea     dx,font_error_msg
                   mov     ah,9
                   int     21h             ;printing error message
                   jmp     finished
                                           ;
read_font_file:    mov     bx,ax           ;get file handle
                   push    ds              ;save data seg
                   mov     ax,ramfont_seg  ;load ramfont segment
                   mov     ds,ax
                   xor     dx,dx
                   mov     cx,lramfont     ;48k bytes
                   mov     ah,3fh          ;read file
                   int     21h
                   mov     ah,3eh          ;close file
                   int     21h
                   pop     ds
                                           ;
screen_on:         mov     dx,dmc_port     ;turn screen on
                   mov     al,scrn_on_nb   ;blink enable clear to use bold
                   out     dx,al
                                           ;
recycle:           @call   endc            ;end of critical region
                   @call   pause           ;give up rest of time slice
                   jmp     begin_monitor   ;go back for another check

finished:          ret
monitor            ENDP
                                           ;
;********************************************************************
;             ARGV PROCEDURE to get command line arguments
;********************************************************************
; Call with:  ES:BX = command line address
;                     (implicit: ES=PSP segment)
;             AX    = argument number (0 based)
;
; Returns:    ES:BX = argument address
;             AX    = argument length
;                     (0=argument not found)
;             Other registers preserved.
;
; If called with AX=0 (argv[0]) and running under
; MS-DOS version 3.0 or later, returns ES:BX pointing
; to program name in environment block and AX=length,
; otherwise returns ES:BX unchanged and AX=0.
;

cr      equ     0dh             ; ASCII carriage return
lf      equ     0ah             ; ASCII line feed
tab     equ     09h             ; ASCII tab
blank   equ     20h             ; ASCII space character
eqsign  equ     3dh             ; ASCII equal sign

argv    proc    near
        or      ax,ax           ; is it argument 0?
        jz      argv8           ; yes, jump to get program name

        xor     ah,ah           ; initialize argument counter

argv1:  mov     cx,-1           ; set flag = outside argument

argv2:  inc     bx              ; point to next character
        cmp     byte ptr es:[bx],cr
        je      argv7           ; exit if carriage return
        cmp     byte ptr es:[bx],blank
        je      argv1           ; outside argument if ASCII blank
        cmp     byte ptr es:[bx],tab
        je      argv1           ; outside argument if ASCII tab
        cmp     byte ptr es:[bx],eqsign
        je      argv1           ; outside argument if ASCII equal sign

                                ; if not blank, tab or equal sign...
        jcxz    argv2           ; jump if already inside argument

        inc     ah              ; else count arguments found
        cmp     ah,al           ; is this the one we're looking for?
        je      argv4           ; yes, go find its length
        not     cx              ; no, set flag = inside argument
        jmp     argv2           ; and look at next character

argv4:                          ; found desired argument, now
                                ; determine its length...
        mov     ax,bx           ; save param. starting address

argv5:  inc     bx              ; point to next character
        cmp     byte ptr es:[bx],cr
        je      argv6           ; found end if carriage return
        cmp     byte ptr es:[bx],blank
        je      argv6           ; found end if ASCII blank
        cmp     byte ptr es:[bx],tab
        je      argv6           ; found end if ASCII tab
        cmp     byte ptr es:[bx],eqsign
        jne     argv5           ; found end if ASCII equal sign

argv6:  xchg    bx,ax           ; set ES:BX = argument address
        sub     ax,bx           ; and AX = argument length
        jmp     argvx           ; return to caller

argv7:  xor     ax,ax           ; set AX = 0, argument not found
        jmp     argvx           ; return to caller

argv8:                          ; special handling for argv=0
        mov     ax,3000h        ; check if DOS 3.0 or later
        int     21h             ; (force AL=0 in case DOS 1)
        cmp     al,3
        jb      argv7           ; DOS 1 or 2, return null param.
        mov     es,es:[2ch]     ; get environment segment from PSP
        xor     di,di           ; find the program name by
        xor     al,al           ; first skipping over all the
        mov     cx,-1           ; environment variables...
        cld
argv9:  repne scasb             ; scan for double null (can't use
        scasb                   ; (SCASW since might be odd addr.)
        jne     argv9           ; loop if it was a single null
        add     di,2            ; skip count word in environment
        mov     bx,di           ; save program name address
        mov     cx,-1           ; now find its length...
        repne scasb             ; scan for another null byte
        not     cx              ; convert CX to length
        dec     cx
        mov     ax,cx           ; return length in AX

argvx:                          ; common exit point
        ret                     ; return to caller

argv    endp

;****************************************************************************
;                       INITIALIZATION PROCEDURE
;****************************************************************************
                                           ;
                                           ;
load_monitor       proc    near
                                           ;
                   assume  cs:code_seg, ds:nothing, es:nothing
                                           ;
                   mov     PSP_seg,es      ;save PSP
                                           ;
;make sure it is GB112
;       Performs two tests - first, makes sure that a Hercules
;       Graphics Card is present and active, then checks that it is a GB112.
        mov     ah,15                   ; check current video state
        int     10h

        cmp     al,7                    ; is it 80 x 25 "B&W" Card
        je      mono_ok                 ; yes - continue

        mov     dx,offset fail_message1 ; no - quit
        jmp     test_failed

mono_ok:
        mov     dx,status_port          ; record state
        in      al,dx
        and     al,80h                  ; save bit 7 for test
        mov     ah,al

        mov     cx,8000h

examine:
        in      al,dx                   ; take another reading
        and     al,80h                  ; again save bit seven
        cmp     al,ah
        jne     hgc                     ; if bit 7 changes, then it
        loop    examine                 ; is a Hercules Graphics Card

        lea     dx,fail_message1
        jmp     test_failed             ; after this long it must be
                                        ; something else - quit.

hgc:    in      al,dx                   ; test for GB112
        and     al,id_mask              ; clear all but bits 4 and 5

        cmp     al,id_code              ; test ID bits
        jne     t_f_1                   ; exit if failed - not a GB112

        jmp     chk_dv

t_f_1:
        lea     dx,fail_message2
        jmp     test_failed
                                           ;
;make sure this is a DESQview environment
chk_dv:            @call   dvpresent
                   test    ax,ax
                   jnz     install
                   lea     dx,fail_message3
                   jmp     test_failed
                                           ;
install:           mov     dx,config_port  ; set the HGC+ to FULL
                   mov     al,3
                   out     dx,al
                                           ;
                                           ;
;DESQview actions
;setup to open main application
                   mov     bx,cmd_tail     ;setup to get command line arg
                   mov     es,PSP_seg
                   mov     ax,1            ;get first argument
                   call    argv
                   mov     di,ax
                   mov     byte ptr es:[di+bx],0 ;make filespec ASCIIZ
                   mov     al,byte ptr es:[di+bx+1] ;get the second argument
                   sub     al,30h          ;convert to binary integer
                   mov     ramfont,al      ;save it
                   mov     al,byte ptr es:[di+bx+3] ;get the third argument
                   sub     al,30h          ;convert to binary integer
                   mov     font_file,al    ;save it
                   mov     al,byte ptr es:[di+bx+5] ;get fourth argument
                   mov     open_window,al  ;save it
                   push    ds              ;save DS
                   mov     ax,es           ;point to ASCIIZ filespec
                   mov     ds,ax
                   mov     dx,bx
                   mov     ax,3d00h        ;open file for reading
                   int     21h
                   pop     ds
                   jnc     read_file
                   lea     dx,file_error_msg
                   jmp     test_failed
                                           ;
read_file:         mov     bx,ax           ;get file handle
                   lea     dx,pif_buffer
                   mov     cx,416          ;DV 2.00 PIFs are 416 bytes
                   mov     ah,3fh          ;read file
                   int     21h
                   mov     ah,3eh          ;close file
                   int     21h
                                           ;
                   cmp     open_window,'Y' ;is application window to be opened
                   je      open_win        ;yes, go do it
                                           ;
;no, so must find out what window at 0,0
                   xor     bx,bx           ;check location 0,0
                   mov     es,bx           ;start at top of list
                   @call   locate
                   mov     ax,es           ;get high order word of handle
                   test    ax,ax           ;is it 0
                   jz      no_win          ;yes, no window found, so error
                   mov     main_win[2],ax  ;save handle
                   jmp     get_path

no_win:            lea     dx,win_error_msg
                   jmp     test_failed

;open new process for main application
open_win:          lea     di,pif_buffer
                   mov     bx,416
                   @call   newproc         ;open main application in DV
                   mov     main_win[2],bx  ;get handle - high order word
                                           ;
;get path to WordPerfect 5.0 from PIF
get_path:          push    ds              ;point to PIF buffer
                   pop     es
                   lea     di,pif_buffer.path_cmd ;point to command field
                   mov     cx,64           ;max length of command field
                   xor     ax,ax           ;looking for null byte
                   cld                     ;look forward
                   repne   scasb
                   mov     ax,64
                   sub     ax,cx
                   mov     cx,ax           ;setup to find last \
                   mov     al,'\'
                   std                     ;look back
                   repne   scasb
                   jcxz    no_path
                   inc     di              ;point to byte beyond last \
                   inc     di
                   cmp     font_file,2     ;is it font_file2
                   je      get_2           ;yes, go get it
                   lea     si,font_file3   ;no, point to font_file3
                   mov     cx,lfont_file3  ;get length
                   jmp     move_file

no_path:           lea     dx,path_error_msg
                   jmp     test_failed

get_2:             lea     si,font_file2   ;point to font_file2
                   mov     cx,lfont_file2

move_file:         cld                     ;store forward
                   rep     movsb           ;store path

                                           ;
;make sure the font file is there
                   lea     dx,pif_buffer.path_cmd ;point to font_file
                   mov     ax,3d00h         ;open file for reading
                   int     21h
                   jnc     close_font_file
                   lea     dx,font_error_msg
                   jmp     test_failed

close_font_file:   mov     bx,ax
                   mov     ax,3e00h
                   int     21h
                                           ;
;direct window manager to notify you of all video mode changes
                   lea     di,mode_chg_msg
                   xor     dx,dx
                   mov     cx,lmode_chg_msg
                   @push   dsdi
                   @push   dxcx
                   @send   write,me
                                           ;
                   lea     dx,install_message
                   mov     ah,9
                   int     21h             ;report installed
                                           ;
                   call    monitor         ;go wait on messages
                                           ;
                   mov     ax,4c00h        ;normal exit
                   int     21h             ;
                                           ;
test_failed:       mov     ah,9            ;error exit
                   int     21h             ;printing error message
                   lea     dx,pause_msg
                   mov     ah,9
                   int     21h
                   mov     ah,1            ;wait for keystroke
                   int     21h
                   mov     ax,4c01h        ;and returning errorlevel
                   int     21h             ;

load_monitor       endp
                                           ;
                                           ;
code_seg           ends
                                           ;
                   end     first           ;begin at first
